@@ -1,4 +1,4 @@
-package openapi3filter_test
+package openapi3filter
 
 import (
 	"bytes"
@@ -6,7 +6,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
 	"net/url"
@@ -14,9 +13,10 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/getkin/kin-openapi/openapi3"
-	"github.com/getkin/kin-openapi/openapi3filter"
 	"github.com/stretchr/testify/require"
+
+	"github.com/getkin/kin-openapi/openapi3"
+	legacyrouter "github.com/getkin/kin-openapi/routers/legacy"
 )
 
 type ExampleRequest struct {
@@ -45,7 +45,7 @@ func TestFilter(t *testing.T) {
 	complexArgSchema.Required = []string{"name", "id"}
 
 	// Declare router
-	swagger := &openapi3.Swagger{
+	doc := &openapi3.T{
 		OpenAPI: "3.0.0",
 		Info: &openapi3.Info{
 			Title:   "MyAPI",
@@ -56,15 +56,16 @@ func TestFilter(t *testing.T) {
 				URL: "http://example.com/api/",
 			},
 		},
-		Paths: openapi3.Paths{
-			"/prefix/{pathArg}/suffix": &openapi3.PathItem{
+		Paths: openapi3.NewPaths(
+			openapi3.WithPath("/prefix/{pathArg}/suffix", &openapi3.PathItem{
 				Post: &openapi3.Operation{
 					Parameters: openapi3.Parameters{
 						{
 							Value: &openapi3.Parameter{
-								In:     "path",
-								Name:   "pathArg",
-								Schema: openapi3.NewStringSchema().WithMaxLength(2).NewRef(),
+								In:       "path",
+								Name:     "pathArg",
+								Schema:   openapi3.NewStringSchema().WithMaxLength(2).NewRef(),
+								Required: true,
 							},
 						},
 						{
@@ -104,7 +105,7 @@ func TestFilter(t *testing.T) {
 								).NewRef(),
 							},
 						},
-						// TODO(decode not): handle decoding "not" JSON Schema
+						// TODO(decode not): handle decoding "not" Schema
 						// {
 						// 	Value: &openapi3.Parameter{
 						// 		In:   "query",
@@ -135,9 +136,9 @@ func TestFilter(t *testing.T) {
 					},
 					Responses: openapi3.NewResponses(),
 				},
-			},
+			}),
 
-			"/issue151": &openapi3.PathItem{
+			openapi3.WithPath("/issue151", &openapi3.PathItem{
 				Get: &openapi3.Operation{
 					Responses: openapi3.NewResponses(),
 				},
@@ -151,36 +152,40 @@ func TestFilter(t *testing.T) {
 						},
 					},
 				},
-			},
-		},
+			}),
+		),
 	}
 
-	router := openapi3filter.NewRouter().WithSwagger(swagger)
-	expectWithDecoder := func(req ExampleRequest, resp ExampleResponse, decoder openapi3filter.ContentParameterDecoder) error {
+	err := doc.Validate(context.Background())
+	require.NoError(t, err)
+	router, err := legacyrouter.NewRouter(doc)
+	require.NoError(t, err)
+	expectWithDecoder := func(req ExampleRequest, resp ExampleResponse, decoder ContentParameterDecoder) error {
 		t.Logf("Request: %s %s", req.Method, req.URL)
-		httpReq, _ := http.NewRequest(req.Method, req.URL, marshalReader(req.Body))
-		httpReq.Header.Set("Content-Type", req.ContentType)
+		httpReq, err := http.NewRequest(req.Method, req.URL, marshalReader(req.Body))
+		require.NoError(t, err)
+		httpReq.Header.Set(headerCT, req.ContentType)
 
 		// Find route
-		route, pathParams, err := router.FindRoute(httpReq.Method, httpReq.URL)
+		route, pathParams, err := router.FindRoute(httpReq)
 		require.NoError(t, err)
 
 		// Validate request
-		requestValidationInput := &openapi3filter.RequestValidationInput{
+		requestValidationInput := &RequestValidationInput{
 			Request:      httpReq,
 			PathParams:   pathParams,
 			Route:        route,
 			ParamDecoder: decoder,
 		}
-		if err := openapi3filter.ValidateRequest(context.TODO(), requestValidationInput); err != nil {
+		if err := ValidateRequest(context.Background(), requestValidationInput); err != nil {
 			return err
 		}
 		t.Logf("Response: %d", resp.Status)
-		responseValidationInput := &openapi3filter.ResponseValidationInput{
+		responseValidationInput := &ResponseValidationInput{
 			RequestValidationInput: requestValidationInput,
 			Status:                 resp.Status,
 			Header: http.Header{
-				"Content-Type": []string{
+				headerCT: []string{
 					resp.ContentType,
 				},
 			},
@@ -190,27 +195,23 @@ func TestFilter(t *testing.T) {
 			require.NoError(t, err)
 			responseValidationInput.SetBodyBytes(data)
 		}
-		err = openapi3filter.ValidateResponse(context.TODO(), responseValidationInput)
+		err = ValidateResponse(context.Background(), responseValidationInput)
 		require.NoError(t, err)
-		return err
+		return nil
 	}
 	expect := func(req ExampleRequest, resp ExampleResponse) error {
 		return expectWithDecoder(req, resp, nil)
 	}
 
-	var err error
-	var req ExampleRequest
-	var resp ExampleResponse
-	resp = ExampleResponse{
+	resp := ExampleResponse{
 		Status: 200,
 	}
-	// Test paths
 
-	req = ExampleRequest{
+	// Test paths
+	req := ExampleRequest{
 		Method: "POST",
 		URL:    "http://example.com/api/prefix/v/suffix",
 	}
-
 	err = expect(req, resp)
 	require.NoError(t, err)
 
@@ -220,7 +221,7 @@ func TestFilter(t *testing.T) {
 		URL:    "http://example.com/api/prefix/EXCEEDS_MAX_LENGTH/suffix",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	// Test query parameter openapi3filter
 	req = ExampleRequest{
@@ -235,14 +236,14 @@ func TestFilter(t *testing.T) {
 		URL:    "http://example.com/api/prefix/v/suffix?queryArg=EXCEEDS_MAX_LENGTH",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	req = ExampleRequest{
 		Method: "GET",
 		URL:    "http://example.com/api/issue151?par2=par1_is_missing",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	// Test query parameter openapi3filter
 	req = ExampleRequest{
@@ -264,51 +265,44 @@ func TestFilter(t *testing.T) {
 		URL:    "http://example.com/api/prefix/v/suffix?queryArgAnyOf=123",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
-
-	req = ExampleRequest{
-		Method: "POST",
-		URL:    "http://example.com/api/prefix/v/suffix?queryArgOneOf=567",
-	}
-	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	req = ExampleRequest{
 		Method: "POST",
 		URL:    "http://example.com/api/prefix/v/suffix?queryArgOneOf=2017-12-31T11:59:59",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	req = ExampleRequest{
 		Method: "POST",
 		URL:    "http://example.com/api/prefix/v/suffix?queryArgAllOf=abdfg",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
-	// TODO(decode not): handle decoding "not" JSON Schema
+	// TODO(decode not): handle decoding "not" Schema
 	// req = ExampleRequest{
 	// 	Method: "POST",
 	// 	URL:    "http://example.com/api/prefix/v/suffix?queryArgNot=abdfg",
 	// }
 	// err = expect(req, resp)
-	// require.IsType(t, &openapi3filter.RequestError{}, err)
+	// require.IsType(t, &RequestError{}, err)
 
-	// TODO(decode not): handle decoding "not" JSON Schema
+	// TODO(decode not): handle decoding "not" Schema
 	// req = ExampleRequest{
 	// 	Method: "POST",
 	// 	URL:    "http://example.com/api/prefix/v/suffix?queryArgNot=123",
 	// }
 	// err = expect(req, resp)
-	// require.IsType(t, &openapi3filter.RequestError{}, err)
+	// require.IsType(t, &RequestError{}, err)
 
 	req = ExampleRequest{
 		Method: "POST",
 		URL:    "http://example.com/api/prefix/v/suffix?queryArg=EXCEEDS_MAX_LENGTH",
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	req = ExampleRequest{
 		Method: "POST",
@@ -318,14 +312,14 @@ func TestFilter(t *testing.T) {
 		Status: 200,
 	}
 	err = expect(req, resp)
-	// require.IsType(t, &openapi3filter.ResponseError{}, err)
+	// require.IsType(t, &ResponseError{}, err)
 	require.NoError(t, err)
 
 	// Check that content validation works. This should pass, as ID is short
 	// enough.
 	req = ExampleRequest{
 		Method: "POST",
-		URL:    "http://example.com/api/prefix/v/suffix?contentArg={\"name\":\"bob\", \"id\":\"a\"}",
+		URL:    `http://example.com/api/prefix/v/suffix?contentArg={"name":"bob", "id":"a"}`,
 	}
 	err = expect(req, resp)
 	require.NoError(t, err)
@@ -333,10 +327,10 @@ func TestFilter(t *testing.T) {
 	// Now it should fail due the ID being too long
 	req = ExampleRequest{
 		Method: "POST",
-		URL:    "http://example.com/api/prefix/v/suffix?contentArg={\"name\":\"bob\", \"id\":\"EXCEEDS_MAX_LENGTH\"}",
+		URL:    `http://example.com/api/prefix/v/suffix?contentArg={"name":"bob", "id":"EXCEEDS_MAX_LENGTH"}`,
 	}
 	err = expect(req, resp)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 
 	// Now, repeat the above two test cases using a custom parameter decoder.
 	customDecoder := func(param *openapi3.Parameter, values []string) (interface{}, *openapi3.Schema, error) {
@@ -348,7 +342,7 @@ func TestFilter(t *testing.T) {
 
 	req = ExampleRequest{
 		Method: "POST",
-		URL:    "http://example.com/api/prefix/v/suffix?contentArg2={\"name\":\"bob\", \"id\":\"a\"}",
+		URL:    `http://example.com/api/prefix/v/suffix?contentArg2={"name":"bob", "id":"a"}`,
 	}
 	err = expectWithDecoder(req, resp, customDecoder)
 	require.NoError(t, err)
@@ -356,10 +350,10 @@ func TestFilter(t *testing.T) {
 	// Now it should fail due the ID being too long
 	req = ExampleRequest{
 		Method: "POST",
-		URL:    "http://example.com/api/prefix/v/suffix?contentArg2={\"name\":\"bob\", \"id\":\"EXCEEDS_MAX_LENGTH\"}",
+		URL:    `http://example.com/api/prefix/v/suffix?contentArg2={"name":"bob", "id":"EXCEEDS_MAX_LENGTH"}`,
 	}
 	err = expectWithDecoder(req, resp, customDecoder)
-	require.IsType(t, &openapi3filter.RequestError{}, err)
+	require.IsType(t, &RequestError{}, err)
 }
 
 func marshalReader(value interface{}) io.ReadCloser {
@@ -370,7 +364,7 @@ func marshalReader(value interface{}) io.ReadCloser {
 	if err != nil {
 		panic(err)
 	}
-	return ioutil.NopCloser(bytes.NewReader(data))
+	return io.NopCloser(bytes.NewReader(data))
 }
 
 func TestValidateRequestBody(t *testing.T) {
@@ -403,7 +397,7 @@ func TestValidateRequestBody(t *testing.T) {
 		{
 			name:    "required empty",
 			body:    requiredReqBody,
-			wantErr: &openapi3filter.RequestError{RequestBody: requiredReqBody, Err: openapi3filter.ErrInvalidRequired},
+			wantErr: &RequestError{RequestBody: requiredReqBody, Err: ErrInvalidRequired},
 		},
 		{
 			name: "required not empty",
@@ -434,10 +428,10 @@ func TestValidateRequestBody(t *testing.T) {
 		t.Run(tc.name, func(t *testing.T) {
 			req := httptest.NewRequest(http.MethodPost, "/test", tc.data)
 			if tc.mime != "" {
-				req.Header.Set(http.CanonicalHeaderKey("Content-Type"), tc.mime)
+				req.Header.Set(headerCT, tc.mime)
 			}
-			inp := &openapi3filter.RequestValidationInput{Request: req}
-			err := openapi3filter.ValidateRequestBody(context.Background(), inp, tc.body)
+			inp := &RequestValidationInput{Request: req}
+			err := ValidateRequestBody(context.Background(), inp, tc.body)
 
 			if tc.wantErr == nil {
 				require.NoError(t, err)
@@ -453,11 +447,11 @@ func matchReqBodyError(want, got error) bool {
 	if want == got {
 		return true
 	}
-	wErr, ok := want.(*openapi3filter.RequestError)
+	wErr, ok := want.(*RequestError)
 	if !ok {
 		return false
 	}
-	gErr, ok := got.(*openapi3filter.RequestError)
+	gErr, ok := got.(*RequestError)
 	if !ok {
 		return false
 	}
@@ -478,8 +472,7 @@ func toJSON(v interface{}) io.Reader {
 	return bytes.NewReader(data)
 }
 
-// TestOperationOrSwaggerSecurity asserts that the swagger's SecurityRequirements are used if no SecurityRequirements are provided for an operation.
-func TestOperationOrSwaggerSecurity(t *testing.T) {
+func TestRootSecurityRequirementsAreUsedIfNotProvidedAtTheOperationLevel(t *testing.T) {
 	// Create the security schemes
 	securitySchemes := []ExampleSecurityScheme{
 		{
@@ -503,59 +496,58 @@ func TestOperationOrSwaggerSecurity(t *testing.T) {
 	tc := []struct {
 		name            string
 		schemes         *[]ExampleSecurityScheme
-		expectedSchemes *[]ExampleSecurityScheme
+		expectedSchemes []ExampleSecurityScheme
 	}{
 		{
 			name:    "/inherited-security",
 			schemes: nil,
-			expectedSchemes: &[]ExampleSecurityScheme{
+			expectedSchemes: []ExampleSecurityScheme{
 				securitySchemes[1],
 			},
 		},
 		{
 			name:            "/overwrite-without-security",
 			schemes:         &[]ExampleSecurityScheme{},
-			expectedSchemes: &[]ExampleSecurityScheme{},
+			expectedSchemes: []ExampleSecurityScheme{},
 		},
 		{
 			name: "/overwrite-with-security",
 			schemes: &[]ExampleSecurityScheme{
 				securitySchemes[0],
 			},
-			expectedSchemes: &[]ExampleSecurityScheme{
+			expectedSchemes: []ExampleSecurityScheme{
 				securitySchemes[0],
 			},
 		},
 	}
 
-	// Create the swagger
-	swagger := &openapi3.Swagger{
+	doc := &openapi3.T{
 		OpenAPI: "3.0.0",
 		Info: &openapi3.Info{
 			Title:   "MyAPI",
 			Version: "0.1",
 		},
-		Paths: map[string]*openapi3.PathItem{},
+		Paths: openapi3.NewPaths(),
 		Security: openapi3.SecurityRequirements{
 			{
 				securitySchemes[1].Name: {},
 			},
 		},
-		Components: openapi3.Components{
+		Components: &openapi3.Components{
 			SecuritySchemes: map[string]*openapi3.SecuritySchemeRef{},
 		},
 	}
 
 	// Add the security schemes to the components
 	for _, scheme := range securitySchemes {
-		swagger.Components.SecuritySchemes[scheme.Name] = &openapi3.SecuritySchemeRef{
+		doc.Components.SecuritySchemes[scheme.Name] = &openapi3.SecuritySchemeRef{
 			Value: scheme.Scheme,
 		}
 	}
 
-	// Add the paths from the test cases to the swagger's paths
+	// Add the paths from the test cases to the spec's paths
 	for _, tc := range tc {
-		var securityRequirements *openapi3.SecurityRequirements = nil
+		var securityRequirements *openapi3.SecurityRequirements
 		if tc.schemes != nil {
 			tempS := openapi3.NewSecurityRequirements()
 			for _, scheme := range *tc.schemes {
@@ -563,49 +555,44 @@ func TestOperationOrSwaggerSecurity(t *testing.T) {
 			}
 			securityRequirements = tempS
 		}
-		swagger.Paths[tc.name] = &openapi3.PathItem{
+		doc.Paths.Set(tc.name, &openapi3.PathItem{
 			Get: &openapi3.Operation{
 				Security:  securityRequirements,
 				Responses: openapi3.NewResponses(),
 			},
-		}
+		})
 	}
 
-	// Declare the router
-	router := openapi3filter.NewRouter().WithSwagger(swagger)
+	err := doc.Validate(context.Background())
+	require.NoError(t, err)
+	router, err := legacyrouter.NewRouter(doc)
+	require.NoError(t, err)
 
 	// Test each case
 	for _, path := range tc {
 		// Make a map of the schemes and whether they're
-		var schemesValidated *map[*openapi3.SecurityScheme]bool = nil
-		if path.expectedSchemes != nil {
-			temp := make(map[*openapi3.SecurityScheme]bool)
-			schemesValidated = &temp
-			for _, scheme := range *path.expectedSchemes {
-				(*schemesValidated)[scheme.Scheme] = false
-			}
+		schemesValidated := make(map[*openapi3.SecurityScheme]bool)
+		for _, scheme := range path.expectedSchemes {
+			schemesValidated[scheme.Scheme] = false
 		}
 
 		// Create the request
 		emptyBody := bytes.NewReader(make([]byte, 0))
-		pathURL, err := url.Parse(path.name)
+		httpReq := httptest.NewRequest(http.MethodGet, path.name, emptyBody)
+		route, _, err := router.FindRoute(httpReq)
 		require.NoError(t, err)
-		route, _, err := router.FindRoute(http.MethodGet, pathURL)
-		require.NoError(t, err)
-		req := openapi3filter.RequestValidationInput{
-			Request: httptest.NewRequest(http.MethodGet, path.name, emptyBody),
+		req := RequestValidationInput{
+			Request: httpReq,
 			Route:   route,
-			Options: &openapi3filter.Options{
-				AuthenticationFunc: func(c context.Context, input *openapi3filter.AuthenticationInput) error {
-					if schemesValidated != nil {
-						if validated, ok := (*schemesValidated)[input.SecurityScheme]; ok {
-							if validated {
-								t.Fatalf("The path %q had the schemes %v named %q validated more than once",
-									path.name, input.SecurityScheme, input.SecuritySchemeName)
-							}
-							(*schemesValidated)[input.SecurityScheme] = true
-							return nil
+			Options: &Options{
+				AuthenticationFunc: func(ctx context.Context, input *AuthenticationInput) error {
+					if validated, ok := schemesValidated[input.SecurityScheme]; ok {
+						if validated {
+							t.Fatalf("The path %q had the schemes %v named %q validated more than once",
+								path.name, input.SecurityScheme, input.SecuritySchemeName)
 						}
+						schemesValidated[input.SecurityScheme] = true
+						return nil
 					}
 
 					t.Fatalf("The path %q had the schemes %v named %q",
@@ -617,12 +604,12 @@ func TestOperationOrSwaggerSecurity(t *testing.T) {
 		}
 
 		// Validate the request
-		err = openapi3filter.ValidateRequest(context.TODO(), &req)
+		err = ValidateRequest(context.Background(), &req)
 		require.NoError(t, err)
 
-		for securityRequirement, validated := range *schemesValidated {
+		for securityRequirement, validated := range schemesValidated {
 			if !validated {
-				t.Fatalf("The security requirement %v was exepected to be validated but wasn't",
+				t.Fatalf("The security requirement %v was unexpected to be validated but wasn't",
 					securityRequirement)
 			}
 		}
@@ -662,22 +649,21 @@ func TestAnySecurityRequirementMet(t *testing.T) {
 		},
 	}
 
-	// Create the swagger
-	swagger := openapi3.Swagger{
+	doc := openapi3.T{
 		OpenAPI: "3.0.0",
 		Info: &openapi3.Info{
 			Title:   "MyAPI",
 			Version: "0.1",
 		},
-		Paths: map[string]*openapi3.PathItem{},
-		Components: openapi3.Components{
+		Paths: openapi3.NewPaths(),
+		Components: &openapi3.Components{
 			SecuritySchemes: map[string]*openapi3.SecuritySchemeRef{},
 		},
 	}
 
-	// Add the security schemes to the swagger's components
+	// Add the security schemes to the spec's components
 	for schemeName := range schemes {
-		swagger.Components.SecuritySchemes[schemeName] = &openapi3.SecuritySchemeRef{
+		doc.Components.SecuritySchemes[schemeName] = &openapi3.SecuritySchemeRef{
 			Value: &openapi3.SecurityScheme{
 				Type:   "http",
 				Scheme: "basic",
@@ -685,7 +671,7 @@ func TestAnySecurityRequirementMet(t *testing.T) {
 		}
 	}
 
-	// Add the paths to the swagger
+	// Add the paths to the spec
 	for _, tc := range tc {
 		// Create the security requirements from the test cases's schemes
 		securityRequirements := openapi3.NewSecurityRequirements()
@@ -694,16 +680,18 @@ func TestAnySecurityRequirementMet(t *testing.T) {
 		}
 
 		// Create the path with the security requirements
-		swagger.Paths[tc.name] = &openapi3.PathItem{
+		doc.Paths.Set(tc.name, &openapi3.PathItem{
 			Get: &openapi3.Operation{
 				Security:  securityRequirements,
 				Responses: openapi3.NewResponses(),
 			},
-		}
+		})
 	}
 
-	// Create the router
-	router := openapi3filter.NewRouter().WithSwagger(&swagger)
+	err := doc.Validate(context.Background())
+	require.NoError(t, err)
+	router, err := legacyrouter.NewRouter(&doc)
+	require.NoError(t, err)
 
 	// Create the authentication function
 	authFunc := makeAuthFunc(schemes)
@@ -712,17 +700,18 @@ func TestAnySecurityRequirementMet(t *testing.T) {
 		// Create the request input for the path
 		tcURL, err := url.Parse(tc.name)
 		require.NoError(t, err)
-		route, _, err := router.FindRoute(http.MethodGet, tcURL)
+		httpReq := httptest.NewRequest(http.MethodGet, tcURL.String(), nil)
+		route, _, err := router.FindRoute(httpReq)
 		require.NoError(t, err)
-		req := openapi3filter.RequestValidationInput{
+		req := RequestValidationInput{
 			Route: route,
-			Options: &openapi3filter.Options{
+			Options: &Options{
 				AuthenticationFunc: authFunc,
 			},
 		}
 
 		// Validate the security requirements
-		err = openapi3filter.ValidateSecurityRequirements(context.TODO(), &req, *route.Operation.Security)
+		err = ValidateSecurityRequirements(context.Background(), &req, *route.Operation.Security)
 
 		// If there should have been an error
 		if tc.error {
@@ -757,22 +746,21 @@ func TestAllSchemesMet(t *testing.T) {
 		},
 	}
 
-	// Create the swagger
-	swagger := openapi3.Swagger{
+	doc := openapi3.T{
 		OpenAPI: "3.0.0",
 		Info: &openapi3.Info{
 			Title:   "MyAPI",
 			Version: "0.1",
 		},
-		Paths: map[string]*openapi3.PathItem{},
-		Components: openapi3.Components{
+		Paths: openapi3.NewPaths(),
+		Components: &openapi3.Components{
 			SecuritySchemes: map[string]*openapi3.SecuritySchemeRef{},
 		},
 	}
 
-	// Add the security schemes to the swagger's components
+	// Add the security schemes to the spec's components
 	for schemeName := range schemes {
-		swagger.Components.SecuritySchemes[schemeName] = &openapi3.SecuritySchemeRef{
+		doc.Components.SecuritySchemes[schemeName] = &openapi3.SecuritySchemeRef{
 			Value: &openapi3.SecurityScheme{
 				Type:   "http",
 				Scheme: "basic",
@@ -780,7 +768,7 @@ func TestAllSchemesMet(t *testing.T) {
 		}
 	}
 
-	// Add the paths to the swagger
+	// Add the paths to the spec
 	for _, tc := range tc {
 		// Create the security requirement for the path
 		securityRequirement := openapi3.SecurityRequirement{}
@@ -792,18 +780,20 @@ func TestAllSchemesMet(t *testing.T) {
 			}
 		}
 
-		swagger.Paths[tc.name] = &openapi3.PathItem{
+		doc.Paths.Set(tc.name, &openapi3.PathItem{
 			Get: &openapi3.Operation{
 				Security: &openapi3.SecurityRequirements{
 					securityRequirement,
 				},
 				Responses: openapi3.NewResponses(),
 			},
-		}
+		})
 	}
 
-	// Create the router from the swagger
-	router := openapi3filter.NewRouter().WithSwagger(&swagger)
+	err := doc.Validate(context.Background())
+	require.NoError(t, err)
+	router, err := legacyrouter.NewRouter(&doc)
+	require.NoError(t, err)
 
 	// Create the authentication function
 	authFunc := makeAuthFunc(schemes)
@@ -812,17 +802,18 @@ func TestAllSchemesMet(t *testing.T) {
 		// Create the request input for the path
 		tcURL, err := url.Parse(tc.name)
 		require.NoError(t, err)
-		route, _, err := router.FindRoute(http.MethodGet, tcURL)
+		httpReq := httptest.NewRequest(http.MethodGet, tcURL.String(), nil)
+		route, _, err := router.FindRoute(httpReq)
 		require.NoError(t, err)
-		req := openapi3filter.RequestValidationInput{
+		req := RequestValidationInput{
 			Route: route,
-			Options: &openapi3filter.Options{
+			Options: &Options{
 				AuthenticationFunc: authFunc,
 			},
 		}
 
 		// Validate the security requirements
-		err = openapi3filter.ValidateSecurityRequirements(context.TODO(), &req, *route.Operation.Security)
+		err = ValidateSecurityRequirements(context.Background(), &req, *route.Operation.Security)
 
 		// If there should have been an error
 		if tc.error {
@@ -836,8 +827,8 @@ func TestAllSchemesMet(t *testing.T) {
 // makeAuthFunc creates an authentication function that accepts the given valid schemes.
 // If an invalid or unknown scheme is encountered, an error is returned by the returned function.
 // Otherwise the return value of the returned function is nil.
-func makeAuthFunc(schemes map[string]bool) func(c context.Context, input *openapi3filter.AuthenticationInput) error {
-	return func(c context.Context, input *openapi3filter.AuthenticationInput) error {
+func makeAuthFunc(schemes map[string]bool) func(ctx context.Context, input *AuthenticationInput) error {
+	return func(ctx context.Context, input *AuthenticationInput) error {
 		// If the scheme is valid and present in the schemes
 		valid, present := schemes[input.SecuritySchemeName]
 		if valid && present {
